/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_LINE_H
#define DPC_IMPL_TYPE_LINE_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint32.h"
#include "impl_type_uint8.h"
#include "impl_type_float.h"
#include "dpc/impl_type_vector_line_point.h"

namespace dpc {
struct line {
    ::UInt32 id{ 0U };
    ::UInt8 line_type{ 0U };
    ::UInt8 direction{ 0U };
    ::UInt8 color{ 0U };
    ::UInt8 position_type{ 0U };
    ::Float width{ 0.0F };
    ::Float start_offset{ 0.0F };
    ::Float end_offset{ 0.0F };
    ::dpc::vector_line_point points;

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(id);
        fun(line_type);
        fun(direction);
        fun(color);
        fun(position_type);
        fun(width);
        fun(start_offset);
        fun(end_offset);
        fun(points);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(id);
        fun(line_type);
        fun(direction);
        fun(color);
        fun(position_type);
        fun(width);
        fun(start_offset);
        fun(end_offset);
        fun(points);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("id", id);
        fun("line_type", line_type);
        fun("direction", direction);
        fun("color", color);
        fun("position_type", position_type);
        fun("width", width);
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
        fun("points", points);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("id", id);
        fun("line_type", line_type);
        fun("direction", direction);
        fun("color", color);
        fun("position_type", position_type);
        fun("width", width);
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
        fun("points", points);
    }

    friend bool operator==(const ::dpc::line& lhs, const ::dpc::line& rhs) noexcept
    {
        return (lhs.id == rhs.id) && (lhs.line_type == rhs.line_type) && (lhs.direction == rhs.direction) && (lhs.color == rhs.color) && (lhs.position_type == rhs.position_type) && (fabs(static_cast<double>(lhs.width - rhs.width)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.start_offset - rhs.start_offset)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.end_offset - rhs.end_offset)) < DBL_EPSILON) && (lhs.points == rhs.points);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_LINE_H
